@@ -23,6 +23,7 @@ import gtweak
 from gtweak.tweakmodel import TweakModel
 from gtweak.tweakview import TweakView
 
+
 class MainWindow:
     def __init__(self):
         builder = Gtk.Builder()
@@ -31,10 +32,7 @@ class MainWindow:
 
         filename = os.path.join(gtweak.PKG_DATA_DIR, 'shell.ui')
         builder.add_from_file(filename)
-        
-        toolbar = builder.get_object('toolbar')
-        toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
-        
+
         model = TweakModel()
         view = TweakView(
                     builder,
@@ -42,4 +40,3 @@ class MainWindow:
         builder.get_object('overview_sw').add(view.treeview)
 
         view.run()
-
