@@ -1,3 +1,4 @@
+# This Python file uses the following encoding: utf-8
 # This file is part of gnome-tweak-tool.
 #
 # Copyright (c) 2011 John Stowers
@@ -14,7 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with gnome-tweak-tool.  If not, see <http://www.gnu.org/licenses/>.
-from __future__ import print_function
+
 
 import os.path
 import subprocess
@@ -45,7 +46,7 @@ class _AppChooser(Gtk.Dialog):
         self._all = {}
 
         self.entry = Gtk.SearchEntry(
-                placeholder_text=_("Search Applications..."))
+                placeholder_text=_("Search Applications…"))
         self.entry.set_width_chars(30)
         self.entry.props.activates_default=True
 
@@ -211,7 +212,7 @@ class _StartupTweak(Gtk.ListBoxRow, Tweak):
                         df.get_name(),
                         df.get_description(),
                         **options)
-        
+
         grid = Gtk.Grid(column_spacing=10)
 
         icn = df.get_icon()
@@ -288,6 +289,9 @@ class AutostartListBoxTweakGroup(ListBoxTweakGroup):
                 logging.warning("Error loading desktopfile: %s" % f)
                 continue
 
+            if not AutostartFile(df).is_start_at_login_enabled():
+                continue
+
             sdf = _StartupTweak(df)
             sdf.btn.connect("clicked", self._on_remove_clicked, sdf, df)
             tweaks.append( sdf )
@@ -334,7 +338,7 @@ class AutostartListBoxTweakGroup(ListBoxTweakGroup):
                     str(os.getuid()),'-o','cmd'],
                     stdout=subprocess.PIPE)
         out = cmd.communicate()[0]
-        for l in out.split('\n'):
+        for l in out.decode('utf8').split('\n'):
             exe = l.split(' ')[0]
             if exe and exe[0] != '[': #kernel process
                 exes.append( os.path.basename(exe) )
