@@ -17,16 +17,13 @@
 
 import gtweak
 from gtweak.gshellwrapper import GnomeShellFactory
-from gtweak.tweakmodel import TWEAK_GROUP_WINDOWS, Tweak
-from gtweak.widgets import ListBoxTweakGroup, GSettingsComboEnumTweak, GSettingsComboTweak, GSettingsSwitchTweak, Title, GSettingsSwitchTweakValue, build_label_beside_widget
+from gtweak.tweakmodel import Tweak
+from gtweak.widgets import ListBoxTweakGroup, GSettingsComboEnumTweak, GSettingsComboTweak, GSettingsSwitchTweak, Title, GSettingsSwitchTweakValue, build_label_beside_widget, _GSettingsTweak
 from gtweak.utils import XSettingsOverrides
 import gettext
 
 from gi.repository import Gtk, GLib
 
-_shell = GnomeShellFactory().get_shell()
-_shell_loaded = _shell is not None
-
 class ShowWindowButtons(GSettingsSwitchTweakValue):
 
     def __init__(self, name, value, **options):
@@ -35,31 +32,83 @@ class ShowWindowButtons(GSettingsSwitchTweakValue):
                                            name,
                                            "org.gnome.desktop.wm.preferences",
                                            "button-layout",
-                                           loaded=_shell_loaded,
                                            **options)
     def get_active(self):
         return self.value in self.settings.get_string(self.key_name)
 
     def set_active(self, v):
         val = self.settings.get_string(self.key_name)
-
         (left, colon, right) = val.partition(":")
-        rsplit = right.split(",")
 
-        if v:
-            rsplit.append(self.value)
-        else:
-            rsplit.remove(self.value)
+        if "close" in right:
+            rsplit = right.split(",")
+            rsplit = [x for x in rsplit if x in ['appmenu', 'close', 'maximize', 'minimize']]
 
-        def sort_buttons(x, y):
-            order = ["minimize", "maximize", "close"];
-            if x in order and y in order:
-                return order.index(x) - order.index(y)
+            if v:
+                rsplit.append(self.value)
             else:
-                return 0
-        rsplit.sort(cmp=sort_buttons)
+                rsplit.remove(self.value)
 
-        self.settings.set_string(self.key_name, left + colon + ",".join(rsplit))
+            rsplit.sort(key=lambda x: ["appmenu", "minimize", "maximize", "close"].index(x))
+
+            self.settings.set_string(self.key_name, left + colon + ",".join(rsplit))
+
+        else:
+            rsplit = left.split(",")
+            rsplit = [x for x in rsplit if x in ['appmenu', 'close', 'maximize', 'minimize']]
+
+            if v:
+                rsplit.append(self.value)
+            else:
+                rsplit.remove(self.value)
+
+            rsplit.sort(key=lambda x: ["appmenu", "close", "maximize", "minimize"].index(x))
+
+            self.settings.set_string(self.key_name, ",".join(rsplit) + colon + right)
+
+class PlaceWindowButtons(Gtk.Box, _GSettingsTweak):
+
+    def __init__(self, **options):
+        name = _("Placement")
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
+
+        _GSettingsTweak.__init__(self,
+                                 name,
+                                 "org.gnome.desktop.wm.preferences",
+                                 "button-layout",
+                                 **options)
+
+        box_btn = Gtk.ButtonBox()
+        box_btn.set_layout(Gtk.ButtonBoxStyle.EXPAND)
+
+        # Translators: For RTL languages, this is the "Right" direction since the
+        # interface is flipped
+        btn1 = Gtk.RadioButton.new_with_label_from_widget(None, _("Left"))
+        btn1.set_property("draw-indicator", False)
+
+        btn2 = Gtk.RadioButton.new_from_widget(btn1)
+        # Translators: For RTL languages, this is the "Left" direction since the
+        # interface is flipped
+        btn2.set_label(_("Right"))
+        btn2.set_property("draw-indicator", False)
+
+        val = self.settings.get_string(self.key_name)
+        (left, colon, right) = val.partition(":")
+        if "close" in right:
+           btn2.set_active(True)
+        btn2.connect("toggled", self.on_button_toggled)
+
+        box_btn.pack_start(btn1, True, True, 0)
+        box_btn.pack_start(btn2, True, True, 0)
+
+        build_label_beside_widget(name, box_btn, hbox=self)
+
+    def on_button_toggled(self, v):
+        val = self.settings.get_string(self.key_name)
+        (left, colon, right) = val.partition(":")
+        left = ','.join(list(reversed(left.split(','))))
+        right = ','.join(list(reversed(right.split(','))))
+        self.settings.set_string(self.key_name, right + colon + left)
 
 class WindowScalingFactorTweak(Gtk.Box, Tweak):
     def __init__(self, **options):
@@ -132,23 +181,29 @@ class WindowScalingFactorTweak(Gtk.Box, Tweak):
         self._close()
         self._dialog.destroy()
 
+Focus =  GSettingsComboTweak(_("Focus Mode"), "org.gnome.desktop.wm.preferences", "focus-mode",
+      [("click", _("Click to Focus")), ("sloppy", _("Sloppy")), ("mouse", _("Mouse"))])
+depends_how = lambda x,kn: x.get_string(kn) in ("mouse", "sloppy")
+
 TWEAK_GROUPS = [
-    ListBoxTweakGroup(TWEAK_GROUP_WINDOWS,
-        GSettingsSwitchTweak(_("Attached Modal Dialogs"),"org.gnome.mutter", "attach-modal-dialogs"),
-        GSettingsSwitchTweak(_("Automatically Raise Windows"),"org.gnome.desktop.wm.preferences", "auto-raise"),
-        GSettingsSwitchTweak(_("Resize with Secondary-click"),"org.gnome.desktop.wm.preferences", "resize-with-right-button"),
+    ListBoxTweakGroup(_("Windows"),
+        GSettingsSwitchTweak(_("Attach Modal Dialogs"),"org.gnome.mutter", "attach-modal-dialogs",
+                        desc=_("When on, modal dialog windows are attached to their parent windows, and cannot be moved.")),
+        GSettingsSwitchTweak(_("Resize with Secondary-Click"),"org.gnome.desktop.wm.preferences", "resize-with-right-button"),
         GSettingsComboTweak(_("Window Action Key"),
                         "org.gnome.desktop.wm.preferences",
                         "mouse-button-modifier",
                         [("disabled", _("Disabled")), ("<Alt>", "Alt"), ("<Super>", "Super")]),
-        GSettingsComboEnumTweak(_("Focus Mode"), "org.gnome.desktop.wm.preferences", "focus-mode"),
+        Focus,
+        GSettingsSwitchTweak(_("Raise Windows When Focused"),"org.gnome.desktop.wm.preferences", "auto-raise", depends_on=Focus, depends_how=depends_how),
         Title(_("Titlebar Actions"), "", uid="title-titlebar-actions"),
-        GSettingsComboEnumTweak(_("Double-click"),"org.gnome.desktop.wm.preferences", "action-double-click-titlebar"),
-        GSettingsComboEnumTweak(_("Middle-click"),"org.gnome.desktop.wm.preferences", "action-middle-click-titlebar"),
-        GSettingsComboEnumTweak(_("Secondary-click"),"org.gnome.desktop.wm.preferences", "action-right-click-titlebar"),
+        GSettingsComboEnumTweak(_("Double-Click"),"org.gnome.desktop.wm.preferences", "action-double-click-titlebar"),
+        GSettingsComboEnumTweak(_("Middle-Click"),"org.gnome.desktop.wm.preferences", "action-middle-click-titlebar"),
+        GSettingsComboEnumTweak(_("Secondary-Click"),"org.gnome.desktop.wm.preferences", "action-right-click-titlebar"),
         Title(_("Titlebar Buttons"), "", uid="title-theme"),
         ShowWindowButtons(_("Maximize"), "maximize"),
         ShowWindowButtons(_("Minimize"), "minimize"),
+        PlaceWindowButtons(),
         Title(_("HiDPI"), "", uid="title-hidpi"),
         WindowScalingFactorTweak(),
     )
