@@ -79,7 +79,7 @@ def build_label_beside_widget(txt, *widget, **kwargs):
     #label. By convention this is true in the great majority of cases. Settings that
     #construct their own widgets will need to set this themselves
     lbl.set_mnemonic_widget(widget[-1])
-    
+
     return hbox
 
 def build_combo_box_text(selected, *values):
@@ -126,7 +126,7 @@ def build_tight_button(stock_id):
     provider = Gtk.CssProvider()
     provider.load_from_data(data)
     # 600 = GTK_STYLE_PROVIDER_PRIORITY_APPLICATION
-    button.get_style_context().add_provider(provider, 600) 
+    button.get_style_context().add_provider(provider, 600)
     return button
 
 def adjust_schema_for_overrides(originalSchema, key, options):
@@ -140,9 +140,13 @@ def adjust_schema_for_overrides(originalSchema, key, options):
         overridesSchema = "org.gnome.shell.overrides"
         overridesFile = "org.gnome.shell.gschema.xml"
 
-    if (key in Gio.Settings(schema=overridesSchema).list_keys()):
-        options['schema_filename'] = overridesFile
-        return overridesSchema
+    try:
+        if (key in GSettingsSetting(overridesSchema, schema_filename=overridesFile).list_keys()):
+            options['schema_filename'] = overridesFile
+            return overridesSchema
+    except GSettingsMissingError as e:
+        logging.info("GSetting missing %s" % (e.message))
+
     return originalSchema
 
 
@@ -160,7 +164,7 @@ class _GSettingsTweak(Tweak):
                 name,
                 options.get("description",self.settings.schema_get_description(key_name)),
                 **options)
-        except GSettingsMissingError, e:
+        except GSettingsMissingError as e:
             self.settings = GSettingsFakeSetting()
             Tweak.__init__(self,"","")
             self.loaded = False
@@ -271,20 +275,45 @@ class GSettingsSwitchTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
         w = Gtk.Switch()
         self.settings.bind(key_name, w, "active", Gio.SettingsBindFlags.DEFAULT)
 
-        build_label_beside_widget(name, w, hbox=self)
-        self.widget_for_size_group = None
-
         self.add_dependency_on_tweak(
                 options.get("depends_on"),
                 options.get("depends_how")
         )
 
+        vbox1 = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        vbox1.props.spacing = UI_BOX_SPACING
+        lbl = Gtk.Label(label=name)
+        lbl.props.ellipsize = Pango.EllipsizeMode.END
+        lbl.props.xalign = 0.0
+        vbox1.pack_start(lbl, True, True, 0)
+
+        if options.get("desc"):
+            description = options.get("desc")
+            lbl_desc = Gtk.Label()
+            lbl_desc.props.xalign = 0.0
+            lbl_desc.set_line_wrap(True)
+            lbl_desc.get_style_context().add_class("dim-label")
+            lbl_desc.set_markup("<span size='small'>"+GLib.markup_escape_text(description)+"</span>")
+            vbox1.pack_start(lbl_desc, True, True, 0)
+
+        vbox2 = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        vbox2_upper = Gtk.Box()
+        vbox2_lower = Gtk.Box()
+        vbox2.pack_start(vbox2_upper, True, True, 0)
+        vbox2.pack_start(w, False, False, 0)
+        vbox2.pack_start(vbox2_lower, True, True, 0)
+
+        self.pack_start(vbox1, True, True, 0)
+        self.pack_start(vbox2, False, False, 0)
+        self.widget_for_size_group = None
+
 class GSettingsFontButtonTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
     def __init__(self, name, schema_name, key_name, **options):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, name, schema_name, key_name, **options)
 
         w = Gtk.FontButton()
+        w.set_use_font(True)
         self.settings.bind(key_name, w, "font-name", Gio.SettingsBindFlags.DEFAULT)
         build_label_beside_widget(name, w, hbox=self)
         self.widget_for_size_group = w
@@ -319,7 +348,7 @@ class GSettingsSpinButtonTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
 
         build_label_beside_widget(name, w, hbox=self)
         self.widget_for_size_group = w
-        
+
         self.add_dependency_on_tweak(
                 options.get("depends_on"),
                 options.get("depends_how")
@@ -472,15 +501,16 @@ class DarkThemeSwitcher(Gtk.Box, Tweak):
                        _("Enable the dark theme hint for all the applications in the session"),
                        **options)
 
-        self._gtksettings = GtkSettingsManager()
+        self._gtksettings3 = GtkSettingsManager('3.0')
+        self._gtksettings4 = GtkSettingsManager('4.0')
 
         w = Gtk.Switch()
-        w.set_active(self._gtksettings.get_integer("gtk-application-prefer-dark-theme"))
-		
+        w.set_active(self._gtksettings3.get_integer("gtk-application-prefer-dark-theme"))
+
         title = _("Global Dark Theme")
-        description = _("Applications need to be restarted for change to take effect")
+        description = _("Applications need to be restarted for this change to take place.")
         w.connect("notify::active", self._on_switch_changed)
-        
+
         hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
         hbox.props.spacing = UI_BOX_SPACING
         lbl = Gtk.Label(label=title)
@@ -488,11 +518,12 @@ class DarkThemeSwitcher(Gtk.Box, Tweak):
         lbl.props.xalign = 0.0
         hbox.pack_start(lbl, True, True, 0)
         hbox.pack_start(w, False, False, 0)
-        
+
         lbl_des = Gtk.Label()
         lbl_des.props.xalign = 0.0
-        lbl_des.set_markup("<span size='x-small'>"+GLib.markup_escape_text(description)+"</span>")
-        
+        lbl_des.get_style_context().add_class("dim-label")
+        lbl_des.set_markup("<span size='small'>"+GLib.markup_escape_text(description)+"</span>")
+
         self.pack_start(hbox, False, False, 0)
         self.pack_start(lbl_des, False, False,0)
         self.widget_for_size_group = None
@@ -501,7 +532,9 @@ class DarkThemeSwitcher(Gtk.Box, Tweak):
         active = switch.get_active()
 
         try:
-            self._gtksettings.set_integer("gtk-application-prefer-dark-theme",
+            self._gtksettings3.set_integer("gtk-application-prefer-dark-theme",
+                                          active)
+            self._gtksettings4.set_integer("gtk-application-prefer-dark-theme",
                                           active)
         except:
             self.notify_information(_("Error writing setting"))
@@ -516,7 +549,7 @@ class Title(Gtk.Box, Tweak):
         self.add(widget)
 
 class GSettingsSwitchTweakValue(Gtk.Box, _GSettingsTweak):
-    
+
     def __init__(self, name, schema_name, key_name, **options):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, name, schema_name, key_name, **options)
@@ -528,9 +561,9 @@ class GSettingsSwitchTweakValue(Gtk.Box, _GSettingsTweak):
 
     def _on_toggled(self, sw, pspec):
         self.set_active(sw.get_active())
-    
+
     def set_active(self, v):
         raise NotImplementedError()
-    
+
     def get_active(self):
         raise NotImplementedError()
